@@ -20,12 +20,6 @@
 
 /* This file defines all spatial functions */
 
-#ifdef HAVE_SPATIAL
-
-#ifdef USE_PRAGMA_INTERFACE
-#pragma interface			/* gcc class implementation */
-#endif
-
 #include "sql_type_geom.h"
 #include "item.h"
 #include "gstream.h"
@@ -42,8 +36,9 @@ public:
   Item_geometry_func(THD *thd, Item *a, Item *b, Item *c):
     Item_str_func(thd, a, b, c) {}
   Item_geometry_func(THD *thd, List<Item> &list): Item_str_func(thd, list) {}
-  bool fix_length_and_dec() override;
-  const Type_handler *type_handler() const override { return &type_handler_geometry; }
+  bool fix_length_and_dec(THD *thd) override;
+  const Type_handler *type_handler() const override
+  { return &type_handler_geometry; }
 };
 
 
@@ -57,7 +52,7 @@ protected:
   bool check_arguments() const override
   {
     DBUG_ASSERT(arg_count == 1);
-    return Type_handler_geometry::check_type_geom_or_binary(func_name(),
+    return Type_handler_geometry::check_type_geom_or_binary(func_name_cstring(),
                                                             args[0]);
   }
 public:
@@ -74,7 +69,7 @@ class Item_long_func_args_geometry: public Item_long_func
   bool check_arguments() const override
   {
     DBUG_ASSERT(arg_count == 1);
-    return Type_handler_geometry::check_type_geom_or_binary(func_name(),
+    return Type_handler_geometry::check_type_geom_or_binary(func_name_cstring(),
                                                             args[0]);
   }
 protected:
@@ -95,7 +90,7 @@ protected:
   bool check_arguments() const override
   {
     DBUG_ASSERT(arg_count == 1);
-    return Type_handler_geometry::check_type_geom_or_binary(func_name(),
+    return Type_handler_geometry::check_type_geom_or_binary(func_name_cstring(),
                                                             args[0]);
   }
 public:
@@ -113,7 +108,7 @@ protected:
   bool check_arguments() const override
   {
     DBUG_ASSERT(arg_count >= 1);
-    return Type_handler_geometry::check_type_geom_or_binary(func_name(),
+    return Type_handler_geometry::check_type_geom_or_binary(func_name_cstring(),
                                                             args[0]);
   }
 public:
@@ -135,7 +130,7 @@ protected:
   bool check_arguments() const override
   {
     DBUG_ASSERT(arg_count >= 1);
-    return Type_handler_geometry::check_type_geom_or_binary(func_name(),
+    return Type_handler_geometry::check_type_geom_or_binary(func_name_cstring(),
                                                             args[0]);
   }
 public:
@@ -153,7 +148,7 @@ protected:
   bool check_arguments() const override
   {
     DBUG_ASSERT(arg_count >= 1);
-    return Type_handler_geometry::check_type_geom_or_binary(func_name(),
+    return Type_handler_geometry::check_type_geom_or_binary(func_name_cstring(),
                                                             args[0]);
   }
 public:
@@ -173,7 +168,7 @@ protected:
   bool check_arguments() const override
   {
     DBUG_ASSERT(arg_count >= 2);
-    return Type_handler_geometry::check_types_geom_or_binary(func_name(),
+    return Type_handler_geometry::check_types_geom_or_binary(func_name_cstring(),
                                                              args, 0, 2);
   }
 public:
@@ -192,7 +187,7 @@ protected:
   bool check_arguments() const override
   {
     DBUG_ASSERT(arg_count >= 2);
-    return Type_handler_geometry::check_types_geom_or_binary(func_name(),
+    return Type_handler_geometry::check_types_geom_or_binary(func_name_cstring(),
                                                              args, 0, 2);
   }
 public:
@@ -205,14 +200,18 @@ class Item_func_geometry_from_text: public Item_geometry_func
 {
   bool check_arguments() const override
   {
-    return args[0]->check_type_general_purpose_string(func_name()) ||
+    return args[0]->check_type_general_purpose_string(func_name_cstring()) ||
            check_argument_types_can_return_int(1, MY_MIN(2, arg_count));
   }
 public:
   Item_func_geometry_from_text(THD *thd, Item *a): Item_geometry_func(thd, a) {}
   Item_func_geometry_from_text(THD *thd, Item *a, Item *srid):
     Item_geometry_func(thd, a, srid) {}
-  const char *func_name() const override { return "st_geometryfromtext"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_geometryfromtext") };
+    return name;
+  }
   String *val_str(String *) override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_geometry_from_text>(thd, this); }
@@ -223,14 +222,18 @@ class Item_func_geometry_from_wkb: public Item_geometry_func
   bool check_arguments() const override
   {
     return
-      Type_handler_geometry::check_type_geom_or_binary(func_name(), args[0]) ||
+      Type_handler_geometry::check_type_geom_or_binary(func_name_cstring(), args[0]) ||
       check_argument_types_can_return_int(1, MY_MIN(2, arg_count));
   }
 public:
   Item_func_geometry_from_wkb(THD *thd, Item *a): Item_geometry_func(thd, a) {}
   Item_func_geometry_from_wkb(THD *thd, Item *a, Item *srid):
     Item_geometry_func(thd, a, srid) {}
-  const char *func_name() const override { return "st_geometryfromwkb"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_geometryfromwkb") };
+    return name;
+  }
   String *val_str(String *) override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_geometry_from_wkb>(thd, this); }
@@ -243,7 +246,7 @@ class Item_func_geometry_from_json: public Item_geometry_func
   bool check_arguments() const override
   {
     // TODO: check with Alexey, for better args[1] and args[2] type control
-    return args[0]->check_type_general_purpose_string(func_name()) ||
+    return args[0]->check_type_general_purpose_string(func_name_cstring()) ||
            check_argument_types_traditional_scalar(1, MY_MIN(3, arg_count));
   }
 public:
@@ -252,7 +255,11 @@ public:
     Item_geometry_func(thd, js, opt) {}
   Item_func_geometry_from_json(THD *thd, Item *js, Item *opt, Item *srid):
     Item_geometry_func(thd, js, opt, srid) {}
-  const char *func_name() const override { return "st_geomfromgeojson"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_geomfromgeojson") };
+    return name;
+  }
   String *val_str(String *) override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_geometry_from_json>(thd, this); }
@@ -264,9 +271,13 @@ class Item_func_as_wkt: public Item_str_ascii_func_args_geometry
 public:
   Item_func_as_wkt(THD *thd, Item *a)
    :Item_str_ascii_func_args_geometry(thd, a) {}
-  const char *func_name() const override { return "st_astext"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_astext") };
+    return name;
+  }
   String *val_str_ascii(String *) override;
-  bool fix_length_and_dec() override;
+  bool fix_length_and_dec(THD *thd) override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_as_wkt>(thd, this); }
 };
@@ -276,15 +287,20 @@ class Item_func_as_wkb: public Item_binary_func_args_geometry
 public:
   Item_func_as_wkb(THD *thd, Item *a)
    :Item_binary_func_args_geometry(thd, a) {}
-  const char *func_name() const override { return "st_aswkb"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_aswkb") };
+    return name;
+  }
   String *val_str(String *) override;
-  const Type_handler *type_handler() const override { return &type_handler_long_blob; }
-  bool fix_length_and_dec() override
+  const Type_handler *type_handler() const override
+  { return &type_handler_long_blob; }
+  bool fix_length_and_dec(THD *thd) override
   {
     collation.set(&my_charset_bin);
     decimals=0;
     max_length= (uint32) UINT_MAX32;
-    maybe_null= 1;
+    set_maybe_null();
     return FALSE;
   }
   Item *do_get_copy(THD *thd) const override
@@ -307,8 +323,12 @@ public:
    :Item_str_ascii_func_args_geometry(thd, js, max_dec_digits) {}
   Item_func_as_geojson(THD *thd, Item *js, Item *max_dec_digits, Item *opt)
    :Item_str_ascii_func_args_geometry(thd, js, max_dec_digits, opt) {}
-  const char *func_name() const override { return "st_asgeojson"; }
-  bool fix_length_and_dec() override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_asgeojson") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override;
   String *val_str_ascii(String *) override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_as_geojson>(thd, this); }
@@ -321,12 +341,16 @@ public:
   Item_func_geometry_type(THD *thd, Item *a)
    :Item_str_ascii_func_args_geometry(thd, a) {}
   String *val_str_ascii(String *) override;
-  const char *func_name() const override { return "st_geometrytype"; }
-  bool fix_length_and_dec() override
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_geometrytype") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override
   {
     // "GeometryCollection" is the longest
     fix_length_and_charset(20, default_charset());
-    maybe_null= 1;
+    set_maybe_null();
     return FALSE;
   };
   Item *do_get_copy(THD *thd) const override
@@ -362,7 +386,11 @@ public:
    :Item_geometry_func_args_geometry(thd, a),
     res_heap(8192, sizeof(ch_node))
     {}
-  const char *func_name() const override { return "st_convexhull"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_convexhull") };
+    return name;
+  }
   String *val_str(String *) override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_convexhull>(thd, this); }
@@ -374,7 +402,11 @@ class Item_func_centroid: public Item_geometry_func_args_geometry
 public:
   Item_func_centroid(THD *thd, Item *a)
    :Item_geometry_func_args_geometry(thd, a) {}
-  const char *func_name() const override { return "st_centroid"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_centroid") };
+    return name;
+  }
   String *val_str(String *) override;
   const Type_handler *type_handler() const override
   {
@@ -389,7 +421,11 @@ class Item_func_envelope: public Item_geometry_func_args_geometry
 public:
   Item_func_envelope(THD *thd, Item *a)
    :Item_geometry_func_args_geometry(thd, a) {}
-  const char *func_name() const override { return "st_envelope"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_envelope") };
+    return name;
+  }
   String *val_str(String *) override;
   const Type_handler *type_handler() const override
   {
@@ -427,7 +463,11 @@ class Item_func_boundary: public Item_geometry_func_args_geometry
 public:
   Item_func_boundary(THD *thd, Item *a)
    :Item_geometry_func_args_geometry(thd, a) {}
-  const char *func_name() const override { return "st_boundary"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_boundary") };
+    return name;
+  }
   String *val_str(String *) override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_boundary>(thd, this); }
@@ -442,7 +482,11 @@ public:
   Item_func_point(THD *thd, Item *a, Item *b): Item_geometry_func(thd, a, b) {}
   Item_func_point(THD *thd, Item *a, Item *b, Item *srid):
     Item_geometry_func(thd, a, b, srid) {}
-  const char *func_name() const override { return "point"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("point") };
+    return name;
+  }
   String *val_str(String *) override;
   const Type_handler *type_handler() const override
   {
@@ -458,19 +502,22 @@ class Item_func_spatial_decomp: public Item_geometry_func_args_geometry
 public:
   Item_func_spatial_decomp(THD *thd, Item *a, Item_func::Functype ft):
     Item_geometry_func_args_geometry(thd, a) { decomp_func = ft; }
-  const char *func_name() const override 
+  LEX_CSTRING func_name_cstring() const override
   { 
-    switch (decomp_func)
-    {
+    static LEX_CSTRING startpoint= {STRING_WITH_LEN("st_startpoint") };
+    static LEX_CSTRING endpoint= {STRING_WITH_LEN("st_endpoint") };
+    static LEX_CSTRING exteriorring= {STRING_WITH_LEN("st_exteriorring") };
+    static LEX_CSTRING unknown= {STRING_WITH_LEN("spatial_decomp_unknown") };
+    switch (decomp_func) {
       case SP_STARTPOINT:
-        return "st_startpoint";
+        return startpoint;
       case SP_ENDPOINT:
-        return "st_endpoint";
+        return endpoint;
       case SP_EXTERIORRING:
-        return "st_exteriorring";
+        return exteriorring;
       default:
 	DBUG_ASSERT(0);  // Should never happened
-        return "spatial_decomp_unknown"; 
+        return unknown;
     }
   }
   String *val_str(String *) override;
@@ -484,26 +531,30 @@ class Item_func_spatial_decomp_n: public Item_geometry_func_args_geometry
   bool check_arguments() const override
   {
     return Item_geometry_func_args_geometry::check_arguments() ||
-           args[1]->check_type_can_return_int(func_name());
+           args[1]->check_type_can_return_int(func_name_cstring());
   }
 public:
   Item_func_spatial_decomp_n(THD *thd, Item *a, Item *b, Item_func::Functype ft)
    :Item_geometry_func_args_geometry(thd, a, b),
     decomp_func_n(ft)
   { }
-  const char *func_name() const override 
+  LEX_CSTRING func_name_cstring() const override
   { 
-    switch (decomp_func_n)
-    {
+    static LEX_CSTRING pointn= {STRING_WITH_LEN("st_pointn") };
+    static LEX_CSTRING geometryn= {STRING_WITH_LEN("st_geometryn") };
+    static LEX_CSTRING interiorringn= {STRING_WITH_LEN("st_interiorringn") };
+    static LEX_CSTRING unknown= {STRING_WITH_LEN("spatial_decomp_unknown") };
+
+    switch (decomp_func_n) {
       case SP_POINTN:
-        return "st_pointn";
+        return pointn;
       case SP_GEOMETRYN:
-        return "st_geometryn";
+        return geometryn;
       case SP_INTERIORRINGN:
-        return "st_interiorringn";
+        return interiorringn;
       default:
 	DBUG_ASSERT(0);  // Should never happened
-        return "spatial_decomp_n_unknown"; 
+        return unknown;
     }
   }
   String *val_str(String *) override;
@@ -515,7 +566,7 @@ class Item_func_spatial_collection: public Item_geometry_func
 {
   bool check_arguments() const override
   {
-    return Type_handler_geometry::check_types_geom_or_binary(func_name(), args,
+    return Type_handler_geometry::check_types_geom_or_binary(func_name_cstring(), args,
                                                              0, arg_count);
   }
   enum Geometry::wkbType coll_type; 
@@ -529,13 +580,13 @@ public:
     item_type=it;
   }
   String *val_str(String *) override;
-  bool fix_length_and_dec() override
+  bool fix_length_and_dec(THD *thd) override
   {
-    if (Item_geometry_func::fix_length_and_dec())
+    if (Item_geometry_func::fix_length_and_dec(thd))
       return TRUE;
     for (unsigned int i= 0; i < arg_count; ++i)
     {
-      if (args[i]->is_fixed() && args[i]->field_type() != MYSQL_TYPE_GEOMETRY)
+      if (args[i]->fixed() && args[i]->field_type() != MYSQL_TYPE_GEOMETRY)
       {
         String str;
         args[i]->print(&str, QT_NO_DATA_EXPANSION);
@@ -562,7 +613,11 @@ public:
   {
     return &type_handler_geometrycollection;
   }
-  const char *func_name() const override { return "geometrycollection"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("geometrycollection") };
+    return name;
+  }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_geometrycollection>(thd, this); }
 };
@@ -576,8 +631,13 @@ public:
                                  Geometry::wkb_linestring,
                                  Geometry::wkb_point)
   { }
-  const Type_handler *type_handler() const override { return &type_handler_linestring; }
-  const char *func_name() const override { return "linestring"; }
+  const Type_handler *type_handler() const override
+  { return &type_handler_linestring; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("linestring") };
+    return name;
+  }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_linestring>(thd, this); }
 };
@@ -591,8 +651,13 @@ public:
                                  Geometry::wkb_polygon,
                                  Geometry::wkb_linestring)
   { }
-  const Type_handler *type_handler() const override { return &type_handler_polygon; }
-  const char *func_name() const override { return "polygon"; }
+  const Type_handler *type_handler() const override
+  { return &type_handler_polygon; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("polygon") };
+    return name;
+  }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_polygon>(thd, this); }
 };
@@ -610,7 +675,11 @@ public:
   {
     return &type_handler_multilinestring;
   }
-  const char *func_name() const override { return "multilinestring"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("multilinestring") };
+    return name;
+  }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_multilinestring>(thd, this); }
 };
@@ -628,7 +697,11 @@ public:
   {
     return &type_handler_multipoint;
   }
-  const char *func_name() const override { return "multipoint"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("multipoint") };
+    return name;
+  }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_multipoint>(thd, this); }
 };
@@ -646,7 +719,11 @@ public:
   {
     return &type_handler_multipolygon;
   }
-  const char *func_name() const override { return "multipolygon"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("multipolygon") };
+    return name;
+  }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_multipolygon>(thd, this); }
 };
@@ -668,14 +745,14 @@ protected:
   bool check_arguments() const override
   {
     DBUG_ASSERT(arg_count >= 2);
-    return Type_handler_geometry::check_types_geom_or_binary(func_name(),
+    return Type_handler_geometry::check_types_geom_or_binary(func_name_cstring(),
                                                              args, 0, 2);
   }
 public:
   Item_func_spatial_rel(THD *thd, Item *a, Item *b, enum Functype sp_rel):
     Item_bool_func2_with_rev(thd, a, b), spatial_rel(sp_rel)
   {
-    maybe_null= true;
+    set_maybe_null();
   }
   enum Functype functype() const override { return spatial_rel; }
   enum Functype rev_functype() const override
@@ -709,8 +786,8 @@ public:
   Item_func_spatial_mbr_rel(THD *thd, Item *a, Item *b, enum Functype sp_rel):
     Item_func_spatial_rel(thd, a, b, sp_rel)
   { }
-  longlong val_int() override;
-  const char *func_name() const override;
+  bool val_bool() override;
+  LEX_CSTRING func_name_cstring() const override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_spatial_mbr_rel>(thd, this); }
 };
@@ -725,8 +802,8 @@ public:
   Item_func_spatial_precise_rel(THD *thd, Item *a, Item *b, enum Functype sp_rel):
     Item_func_spatial_rel(thd, a, b, sp_rel), collector()
   { }
-  longlong val_int() override;
-  const char *func_name() const override;
+  bool val_bool() override;
+  LEX_CSTRING func_name_cstring() const override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_spatial_precise_rel>(thd, this); }
 };
@@ -741,14 +818,18 @@ class Item_func_spatial_relate: public Item_bool_func_args_geometry_geometry
   bool check_arguments() const override
   {
     return Item_bool_func_args_geometry_geometry::check_arguments() ||
-           args[2]->check_type_general_purpose_string(func_name());
+           args[2]->check_type_general_purpose_string(func_name_cstring());
   }
 public:
   Item_func_spatial_relate(THD *thd, Item *a, Item *b, Item *matrix):
     Item_bool_func_args_geometry_geometry(thd, a, b, matrix)
   { }
-  longlong val_int() override;
-  const char *func_name() const override { return "st_relate"; }
+  bool val_bool() override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_relate") };
+    return name;
+  }
   bool need_parentheses_in_default() override { return false; }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_spatial_relate>(thd, this); }
@@ -759,12 +840,12 @@ public:
   Spatial operations
 */
 
-class Item_func_spatial_operation: public Item_geometry_func
+class Item_func_spatial_operation final: public Item_geometry_func
 {
   bool check_arguments() const override
   {
     DBUG_ASSERT(arg_count >= 2);
-    return Type_handler_geometry::check_types_geom_or_binary(func_name(),
+    return Type_handler_geometry::check_types_geom_or_binary(func_name_cstring(),
                                                              args, 0, 2);
   }
 public:
@@ -782,7 +863,7 @@ public:
   {}
   virtual ~Item_func_spatial_operation();
   String *val_str(String *) override;
-  const char *func_name() const override;
+  LEX_CSTRING func_name_cstring() const override;
   void print(String *str, enum_query_type query_type) override
   {
     Item_func::print(str, query_type);
@@ -792,12 +873,12 @@ public:
 };
 
 
-class Item_func_buffer: public Item_geometry_func_args_geometry
+class Item_func_buffer final : public Item_geometry_func_args_geometry
 {
   bool check_arguments() const override
   {
     return Item_geometry_func_args_geometry::check_arguments() ||
-           args[1]->check_type_can_return_real(func_name());
+           args[1]->check_type_can_return_real(func_name_cstring());
   }
 protected:
   class Transporter : public Gcalc_operation_transporter
@@ -842,7 +923,11 @@ protected:
 public:
   Item_func_buffer(THD *thd, Item *obj, Item *distance)
    :Item_geometry_func_args_geometry(thd, obj, distance) {}
-  const char *func_name() const override { return "st_buffer"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_buffer") };
+    return name;
+  }
   String *val_str(String *) override;
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_buffer>(thd, this); }
@@ -854,9 +939,14 @@ class Item_func_isempty: public Item_bool_func_args_geometry
 public:
   Item_func_isempty(THD *thd, Item *a)
    :Item_bool_func_args_geometry(thd, a) {}
-  longlong val_int() override;
-  const char *func_name() const override { return "st_isempty"; }
-  bool fix_length_and_dec() override { maybe_null= 1; return FALSE; }
+  bool val_bool() override;
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_isempty") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override
+  { set_maybe_null(); return FALSE; }
   bool need_parentheses_in_default() override { return false; }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_isempty>(thd, this); }
@@ -872,9 +962,13 @@ public:
   Item_func_issimple(THD *thd, Item *a)
    :Item_long_func_args_geometry(thd, a) {}
   longlong val_int() override;
-  const char *func_name() const override { return "st_issimple"; }
-  bool fix_length_and_dec() override { decimals=0; max_length=2; return FALSE; }
-  uint decimal_precision() const override { return 1; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_issimple") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override { decimals=0; max_length=2; return FALSE; }
+  decimal_digits_t decimal_precision() const override { return 1; }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_issimple>(thd, this); }
 };
@@ -885,9 +979,13 @@ public:
   Item_func_isclosed(THD *thd, Item *a)
    :Item_long_func_args_geometry(thd, a) {}
   longlong val_int() override;
-  const char *func_name() const override { return "st_isclosed"; }
-  bool fix_length_and_dec() override { decimals=0; max_length=2; return FALSE; }
-  uint decimal_precision() const override { return 1; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_isclosed") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override { decimals=0; max_length=2; return FALSE; }
+  decimal_digits_t decimal_precision() const override { return 1; }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_isclosed>(thd, this); }
 };
@@ -897,7 +995,11 @@ class Item_func_isring: public Item_func_issimple
 public:
   Item_func_isring(THD *thd, Item *a): Item_func_issimple(thd, a) {}
   longlong val_int() override;
-  const char *func_name() const override { return "st_isring"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_isring") };
+    return name;
+  }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_isring>(thd, this); }
 };
@@ -908,8 +1010,13 @@ public:
   Item_func_dimension(THD *thd, Item *a)
    :Item_long_func_args_geometry(thd, a) {}
   longlong val_int() override;
-  const char *func_name() const override { return "st_dimension"; }
-  bool fix_length_and_dec() override { max_length= 10; maybe_null= 1; return FALSE; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_dimension") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override
+  { max_length= 10; set_maybe_null(); return FALSE; }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_dimension>(thd, this); }
 };
@@ -920,12 +1027,16 @@ class Item_func_x: public Item_real_func_args_geometry
 public:
   Item_func_x(THD *thd, Item *a): Item_real_func_args_geometry(thd, a) {}
   double val_real() override;
-  const char *func_name() const override { return "st_x"; }
-  bool fix_length_and_dec() override
+  LEX_CSTRING func_name_cstring() const override
   {
-    if (Item_real_func::fix_length_and_dec())
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_x") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override
+  {
+    if (Item_real_func::fix_length_and_dec(thd))
       return TRUE;
-    maybe_null= 1;
+    set_maybe_null();
     return FALSE;
   }
   Item *do_get_copy(THD *thd) const override
@@ -938,12 +1049,16 @@ class Item_func_y: public Item_real_func_args_geometry
 public:
   Item_func_y(THD *thd, Item *a): Item_real_func_args_geometry(thd, a) {}
   double val_real() override;
-  const char *func_name() const override { return "st_y"; }
-  bool fix_length_and_dec() override
+  LEX_CSTRING func_name_cstring() const override
   {
-    if (Item_real_func::fix_length_and_dec())
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_y") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override
+  {
+    if (Item_real_func::fix_length_and_dec(thd))
       return TRUE;
-    maybe_null= 1;
+    set_maybe_null();
     return FALSE;
   }
   Item *do_get_copy(THD *thd) const override
@@ -957,8 +1072,13 @@ public:
   Item_func_numgeometries(THD *thd, Item *a)
    :Item_long_func_args_geometry(thd, a) {}
   longlong val_int() override;
-  const char *func_name() const override { return "st_numgeometries"; }
-  bool fix_length_and_dec() override { max_length= 10; maybe_null= 1; return FALSE; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_numgeometries") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override
+  { max_length= 10; set_maybe_null(); return FALSE; }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_numgeometries>(thd, this); }
 };
@@ -970,8 +1090,13 @@ public:
   Item_func_numinteriorring(THD *thd, Item *a)
    :Item_long_func_args_geometry(thd, a) {}
   longlong val_int() override;
-  const char *func_name() const override { return "st_numinteriorrings"; }
-  bool fix_length_and_dec() override { max_length= 10; maybe_null= 1; return FALSE; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_numinteriorrings") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override
+  { max_length= 10; set_maybe_null(); return FALSE; }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_numinteriorring>(thd, this); }
 };
@@ -983,8 +1108,13 @@ public:
   Item_func_numpoints(THD *thd, Item *a)
    :Item_long_func_args_geometry(thd, a) {}
   longlong val_int() override;
-  const char *func_name() const override { return "st_numpoints"; }
-  bool fix_length_and_dec() override { max_length= 10; maybe_null= 1; return FALSE; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_numpoints") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override
+  { max_length= 10; set_maybe_null(); return FALSE; }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_numpoints>(thd, this); }
 };
@@ -995,12 +1125,16 @@ class Item_func_area: public Item_real_func_args_geometry
 public:
   Item_func_area(THD *thd, Item *a): Item_real_func_args_geometry(thd, a) {}
   double val_real() override;
-  const char *func_name() const override { return "st_area"; }
-  bool fix_length_and_dec() override
+  LEX_CSTRING func_name_cstring() const override
   {
-    if (Item_real_func::fix_length_and_dec())
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_area") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override
+  {
+    if (Item_real_func::fix_length_and_dec(thd))
       return TRUE;
-    maybe_null= 1;
+    set_maybe_null();
     return FALSE;
   }
   Item *do_get_copy(THD *thd) const override
@@ -1015,12 +1149,16 @@ public:
   Item_func_glength(THD *thd, Item *a)
    :Item_real_func_args_geometry(thd, a) {}
   double val_real() override;
-  const char *func_name() const override { return "st_length"; }
-  bool fix_length_and_dec() override
+  LEX_CSTRING func_name_cstring() const override
   {
-    if (Item_real_func::fix_length_and_dec())
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_length") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override
+  {
+    if (Item_real_func::fix_length_and_dec(thd))
       return TRUE;
-    maybe_null= 1;
+    set_maybe_null();
     return FALSE;
   }
   Item *do_get_copy(THD *thd) const override
@@ -1034,8 +1172,13 @@ public:
   Item_func_srid(THD *thd, Item *a)
    :Item_long_func_args_geometry(thd, a) {}
   longlong val_int() override;
-  const char *func_name() const override { return "srid"; }
-  bool fix_length_and_dec() override { max_length= 10; maybe_null= 1; return FALSE; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("srid") };
+    return name;
+  }
+  bool fix_length_and_dec(THD *thd) override
+  { max_length= 10; set_maybe_null(); return FALSE; }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_srid>(thd, this); }
 };
@@ -1052,7 +1195,11 @@ public:
   Item_func_distance(THD *thd, Item *a, Item *b)
    :Item_real_func_args_geometry_geometry(thd, a, b) {}
   double val_real() override;
-  const char *func_name() const override { return "st_distance"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_distance") };
+    return name;
+  }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_distance>(thd, this); }
 };
@@ -1066,7 +1213,11 @@ public:
   Item_func_sphere_distance(THD *thd, List<Item> &list):
     Item_real_func(thd, list) {}
   double val_real() override;
-  const char *func_name() const override { return "st_distance_sphere"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_distance_sphere") };
+    return name;
+  }
   Item *do_get_copy(THD *thd) const override
   { return get_item_copy<Item_func_sphere_distance>(thd, this); }
 };
@@ -1081,7 +1232,11 @@ class Item_func_pointonsurface: public Item_geometry_func_args_geometry
 public:
   Item_func_pointonsurface(THD *thd, Item *a)
    :Item_geometry_func_args_geometry(thd, a) {}
-  const char *func_name() const override { return "st_pointonsurface"; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_pointonsurface") };
+    return name;
+  }
   String *val_str(String *) override;
   const Type_handler *type_handler() const override
   {
@@ -1098,8 +1253,12 @@ class Item_func_gis_debug: public Item_long_func
   public:
     Item_func_gis_debug(THD *thd, Item *a): Item_long_func(thd, a)
     { null_value= false; }
-    bool fix_length_and_dec() override { fix_char_length(10); return FALSE; }
-    const char *func_name() const override  { return "st_gis_debug"; }
+    bool fix_length_and_dec(THD *thd) override { fix_char_length(10); return FALSE; }
+  LEX_CSTRING func_name_cstring() const override
+  {
+    static LEX_CSTRING name= {STRING_WITH_LEN("st_gis_debug") };
+    return name;
+  }
     longlong val_int() override;
     bool check_vcol_func_processor(void *arg) override
     {
@@ -1114,10 +1273,4 @@ class Item_func_gis_debug: public Item_long_func
 #define GEOM_NEW(thd, obj_constructor) new (thd->mem_root) obj_constructor
 #define GEOM_TYPE(x) (x)
 
-#else /*HAVE_SPATIAL*/
-
-#define GEOM_NEW(thd, obj_constructor) NULL
-#define GEOM_TYPE(x) NULL
-
-#endif /*HAVE_SPATIAL*/
 #endif /* ITEM_GEOFUNC_INCLUDED */
