@@ -19,11 +19,7 @@
 /* subselect Item */
 
 #include "item.h"
-#ifdef USE_PRAGMA_INTERFACE
-#pragma interface			/* gcc class implementation */
-#endif
-
-#include <queues.h>
+#include "sql_queue.h"
 
 class st_select_lex;
 class st_select_lex_unit;
@@ -50,8 +46,7 @@ class Explain_subq_materialization;
 /* base class for subselects */
 
 class Item_subselect :public Item_result_field,
-                      protected Used_tables_and_const_cache,
-                      protected With_sum_func_cache
+                      protected Used_tables_and_const_cache
 {
   /*
     Set to TRUE if the value is assigned for the subselect
@@ -195,9 +190,6 @@ public:
     return null_value;
   }
   bool fix_fields(THD *thd, Item **ref) override;
-  bool with_subquery() const override { DBUG_ASSERT(fixed); return true; }
-  bool with_sum_func() const override { return m_with_sum_func; }
-  With_sum_func_cache* get_with_sum_func_cache() override { return this; }
   bool mark_as_dependent(THD *thd, st_select_lex *select, Item *item);
   void fix_after_pullout(st_select_lex *new_parent, Item **ref,
                          bool merge) override;
@@ -277,6 +269,7 @@ public:
   {
     return TRUE;
   }
+  bool subselect_table_finder_processor(void *arg) override;
 
   void register_as_with_rec_ref(With_element *with_elem);
   void init_expr_cache_tracker(THD *thd);
@@ -301,7 +294,8 @@ public:
 /* single value subselect */
 
 class Item_cache;
-class Item_singlerow_subselect :public Item_subselect
+class Item_singlerow_subselect :public Item_subselect,
+                                public Type_extra_attributes
 {
 protected:
   Item_cache *value, **row;
@@ -318,17 +312,26 @@ public:
   bool select_transformer(JOIN *join) override;
   void store(uint i, Item* item);
   double val_real() override;
-  longlong val_int () override;
-  String *val_str (String *) override;
+  longlong val_int() override;
+  String *val_str(String *) override;
   bool val_native(THD *thd, Native *) override;
   my_decimal *val_decimal(my_decimal *) override;
   bool val_bool() override;
   bool get_date(THD *thd, MYSQL_TIME *ltime, date_mode_t fuzzydate) override;
   const Type_handler *type_handler() const override;
   bool fix_length_and_dec() override;
+  Type_extra_attributes *type_extra_attributes_addr() override
+  {
+    return this;
+  }
+  const Type_extra_attributes type_extra_attributes() const override
+  {
+    return *this;
+  }
 
   uint cols() const override;
-  Item* element_index(uint i) override { return reinterpret_cast<Item*>(row[i]); }
+  Item* element_index(uint i) override
+  { return reinterpret_cast<Item*>(row[i]); }
   Item** addr(uint i) override { return (Item**)row + i; }
   bool check_cols(uint c) override;
   bool null_inside() override;
@@ -378,7 +381,6 @@ class Item_exists_subselect :public Item_subselect
 protected:
   Item_func_not *upper_not;
   bool value; /* value of this item (boolean: exists/not-exists) */
-  bool abort_on_null;
 
   void init_length_and_dec();
   bool select_prepare_to_be_in();
@@ -402,7 +404,7 @@ public:
 
   Item_exists_subselect(THD *thd_arg, st_select_lex *select_lex);
   Item_exists_subselect(THD *thd_arg):
-    Item_subselect(thd_arg), upper_not(NULL), abort_on_null(0),
+  Item_subselect(thd_arg), upper_not(NULL),
     emb_on_expr_nest(NULL), optimizer(0), exists_transformed(0)
   {}
 
@@ -429,8 +431,6 @@ public:
   bool fix_length_and_dec() override;
   void print(String *str, enum_query_type query_type) override;
   bool select_transformer(JOIN *join) override;
-  void top_level_item() override { abort_on_null=1; }
-  bool is_top_level_item() const override { return abort_on_null; }
   bool exists2in_processor(void *opt_arg) override;
 
   Item* expr_cache_insert_transformer(THD *thd, uchar *unused) override;
@@ -676,7 +676,7 @@ public:
 
   void disable_cond_guard_for_const_null_left_expr(int i)
   {
-    if (left_expr->const_item() && !left_expr->is_expensive())
+    if (left_expr->can_eval_in_optimize())
     {
       if (left_expr->element_index(i)->is_null())
         set_cond_guard_var(i,FALSE);
@@ -835,7 +835,8 @@ public:
   enum enum_engine_type {ABSTRACT_ENGINE, SINGLE_SELECT_ENGINE,
                          UNION_ENGINE, UNIQUESUBQUERY_ENGINE,
                          INDEXSUBQUERY_ENGINE, HASH_SJ_ENGINE,
-                         ROWID_MERGE_ENGINE, TABLE_SCAN_ENGINE};
+                         ROWID_MERGE_ENGINE, TABLE_SCAN_ENGINE,
+                         SINGLE_COLUMN_ENGINE};
 
   subselect_engine(Item_subselect *si,
                    select_result_interceptor *res):
@@ -1008,7 +1009,10 @@ public:
   subselect_uniquesubquery_engine(THD *thd_arg, st_join_table *tab_arg,
 				  Item_in_subselect *subs, Item *where)
     :subselect_engine(subs, 0), tab(tab_arg), cond(where)
-  { DBUG_ASSERT(subs); }
+  {
+    thd= thd_arg;
+    DBUG_ASSERT(subs);
+  }
   ~subselect_uniquesubquery_engine();
   void cleanup() override;
   int prepare(THD *) override;
@@ -1175,6 +1179,9 @@ public:
     PARTIAL_MATCH,  /* Use some partial matching strategy. */
     PARTIAL_MATCH_MERGE, /* Use partial matching through index merging. */
     PARTIAL_MATCH_SCAN,  /* Use partial matching through table scan. */
+    SINGLE_COLUMN_MATCH, /* Use simplified matching when there is only
+                            one field involved. */
+    CONST_RETURN_NULL, /* The result of IN predicate is constant NULL */
     IMPOSSIBLE      /* Subquery materialization is not applicable. */
   };
 
@@ -1195,7 +1202,8 @@ protected:
   ulonglong rowid_merge_buff_size(bool has_non_null_key,
                                   bool has_covering_null_row,
                                   MY_BITMAP *partial_match_key_parts);
-  void choose_partial_match_strategy(bool has_non_null_key,
+  void choose_partial_match_strategy(uint field_count,
+                                     bool has_non_null_key,
                                      bool has_covering_null_row,
                                      MY_BITMAP *partial_match_key_parts);
   bool make_semi_join_conds();
@@ -1291,9 +1299,9 @@ protected:
     Quick sort comparison function that compares two rows of the same table
     indentfied with their row numbers.
   */
-  int cmp_keys_by_row_data(rownum_t a, rownum_t b);
-  static int cmp_keys_by_row_data_and_rownum(Ordered_key *key,
-                                             rownum_t* a, rownum_t* b);
+  int cmp_keys_by_row_data(const rownum_t a, const rownum_t b) const;
+  static int cmp_keys_by_row_data_and_rownum(void *key, const void *a,
+                                             const void *b);
 
   int cmp_key_with_search_key(rownum_t row_num);
 
@@ -1309,23 +1317,23 @@ public:
   /* Initialize a single-column index. */
   bool init(int col_idx);
 
-  uint get_column_count() { return key_column_count; }
-  uint get_keyid() { return keyid; }
-  Field *get_field(uint i)
+  uint get_column_count() const { return key_column_count; }
+  uint get_keyid() const { return keyid; }
+  Field *get_field(uint i) const
   {
     DBUG_ASSERT(i < key_column_count);
     return key_columns[i]->field;
   }
-  rownum_t get_min_null_row() { return min_null_row; }
-  rownum_t get_max_null_row() { return max_null_row; }
+  rownum_t get_min_null_row() const { return min_null_row; }
+  rownum_t get_max_null_row() const { return max_null_row; }
   MY_BITMAP * get_null_key() { return &null_key; }
-  ha_rows get_null_count() { return null_count; }
-  ha_rows get_key_buff_elements() { return key_buff_elements; }
+  ha_rows get_null_count() const { return null_count; }
+  ha_rows get_key_buff_elements() const { return key_buff_elements; }
   /*
     Get the search key element that corresponds to the i-th key part of this
     index.
   */
-  Item *get_search_key(uint i)
+  Item *get_search_key(uint i) const
   {
     return search_key->element_index(key_columns[i]->field->field_index);
   }
@@ -1338,7 +1346,7 @@ public:
   }
 
   bool sort_keys();
-  double null_selectivity();
+  inline double null_selectivity() const;
 
   /*
     Position the current element at the first row that matches the key.
@@ -1366,7 +1374,7 @@ public:
     return FALSE;
   };
   /* Return the current index element. */
-  rownum_t current()
+  rownum_t current() const
   {
     DBUG_ASSERT(key_buff_elements && cur_key_idx < key_buff_elements);
     return key_buff[cur_key_idx];
@@ -1376,7 +1384,7 @@ public:
   {
     bitmap_set_bit(&null_key, (uint)row_num);
   }
-  bool is_null(rownum_t row_num)
+  bool is_null(rownum_t row_num) const
   {
     /*
       Indexes consisting of only NULLs do not have a bitmap buffer at all.
@@ -1392,7 +1400,7 @@ public:
       return FALSE;
     return bitmap_is_set(&null_key, (uint)row_num);
   }
-  void print(String *str);
+  void print(String *str) const;
 };
 
 
@@ -1426,7 +1434,8 @@ protected:
 protected:
   virtual bool partial_match()= 0;
 public:
-  subselect_partial_match_engine(subselect_uniquesubquery_engine *engine_arg,
+  subselect_partial_match_engine(THD *thd,
+                                 subselect_uniquesubquery_engine *engine_arg,
                                  TABLE *tmp_table_arg, Item_subselect *item_arg,
                                  select_result_interceptor *result_arg,
                                  List<Item> *equi_join_conds_arg,
@@ -1504,24 +1513,25 @@ protected:
     Priority queue of Ordered_key indexes, one per NULLable column.
     This queue is used by the partial match algorithm in method exec().
   */
-  QUEUE pq;
+  Queue<Ordered_key> pq;
 protected:
   /*
     Comparison function to compare keys in order of decreasing bitmap
     selectivity.
   */
-  static int cmp_keys_by_null_selectivity(Ordered_key **k1, Ordered_key **k2);
+  static int cmp_keys_by_null_selectivity(const void *k1, const void *k2);
   /*
     Comparison function used by the priority queue pq, the 'smaller' key
     is the one with the smaller current row number.
   */
-  static int cmp_keys_by_cur_rownum(void *arg, uchar *k1, uchar *k2);
+  static int cmp_keys_by_cur_rownum(void *, const void *k1, const void *k2);
 
   bool test_null_row(rownum_t row_num);
   bool exists_complementing_null_row(MY_BITMAP *keys_to_complement);
   bool partial_match() override;
 public:
-  subselect_rowid_merge_engine(subselect_uniquesubquery_engine *engine_arg,
+  subselect_rowid_merge_engine(THD *thd,
+                               subselect_uniquesubquery_engine *engine_arg,
                                TABLE *tmp_table_arg, uint merge_keys_count_arg,
                                bool has_covering_null_row_arg,
                                bool has_covering_null_columns_arg,
@@ -1529,7 +1539,7 @@ public:
                                Item_subselect *item_arg,
                                select_result_interceptor *result_arg,
                                List<Item> *equi_join_conds_arg)
-    :subselect_partial_match_engine(engine_arg, tmp_table_arg,
+    :subselect_partial_match_engine(thd, engine_arg, tmp_table_arg,
                                     item_arg, result_arg, equi_join_conds_arg,
                                     has_covering_null_row_arg,
                                     has_covering_null_columns_arg,
@@ -1548,7 +1558,8 @@ class subselect_table_scan_engine: public subselect_partial_match_engine
 protected:
   bool partial_match() override;
 public:
-  subselect_table_scan_engine(subselect_uniquesubquery_engine *engine_arg,
+  subselect_table_scan_engine(THD *thd,
+                              subselect_uniquesubquery_engine *engine_arg,
                               TABLE *tmp_table_arg, Item_subselect *item_arg,
                               select_result_interceptor *result_arg,
                               List<Item> *equi_join_conds_arg,
@@ -1559,6 +1570,37 @@ public:
   enum_engine_type engine_type() override { return TABLE_SCAN_ENGINE; }
 };
 
+
+/*
+  An engine to handle NULL-aware Materialization for subqueries
+  that compare one column:
+
+    col1 IN (SELECT t2.col2 FROM t2 ...)
+
+  When only one column is used, we need to handle NULL values of
+  col1 and col2 but don't need to perform "partial" matches when only
+  a subset of compared columns is NULL.
+  This allows to save on some data structures.
+*/
+
+class subselect_single_column_match_engine:
+    public subselect_partial_match_engine
+{
+protected:
+  bool partial_match() override;
+public:
+  subselect_single_column_match_engine(THD *thd,
+                              subselect_uniquesubquery_engine *engine_arg,
+                              TABLE *tmp_table_arg, Item_subselect *item_arg,
+                              select_result_interceptor *result_arg,
+                              List<Item> *equi_join_conds_arg,
+                              bool has_covering_null_row_arg,
+                              bool has_covering_null_columns_arg,
+                              uint count_columns_with_nulls_arg);
+  void cleanup() override {}
+  enum_engine_type engine_type() override { return SINGLE_COLUMN_ENGINE; }
+};
+
 /**
   @brief Subquery materialization tracker
 
@@ -1641,6 +1683,10 @@ private:
         return "index_lookup;array merge for partial match";
       case Strategy::PARTIAL_MATCH_SCAN:
         return "index_lookup;full scan for partial match";
+      case Strategy::SINGLE_COLUMN_MATCH:
+        return "null-aware index_lookup";
+      case Strategy::CONST_RETURN_NULL:
+        return "return NULL";
       default:
         return "unsupported";
     }
